c-----------------------------------------------------------------------
c --- UAMAKE -- M3D.DAT Data Preprocessor for UP.DAT Output
c-----------------------------------------------------------------------
c
c --- UAMAKE   Version: 1.53       Level: 090411                    MAIN
c              K. Morrison
c
c     Copyright (c) 2000-2009 by Exponent, Inc.
c
c --- PURPOSE:
c      THIS PROGRAM READS one or more M3D (3D.DAT) FILES and constructs
c      an upper air data file (UP.DAT format) for input to CALMET.
c      Data are extracted from the 3D model grid point(s) nearest the
c      'target' latitude,longitude requested by the user to create the
c      station profile.
c
c --- UPDATES
c
c --- v 1.52 Level 090225  ====> v 1.53 Level 090411 (DGS)
c     1)  Remove standard 12-hour reporting time constraint by
c         adding MTIMES and HRMASK variables
c         Modified: CONTROL.UAS
c                   BLOCK DATA, READCF, SETUP, COMP
c     2)  Use explicit snapshot representation of times, but keep
c         output dataset version 2.1 (all times interpreted as snapshot)
c         Modified: COMP
c
c --- v 1.51 Level 080919  ====> v 1.52 Level 090225 (DGS)
c     1)  Update CALUTILS from V1.0 L010713 to V2.57 L090225
c     2)  Update argument list in call to XTRACTLL
c     3)  Change filename variables from c*70 to c*132
c         Modified: FILNAM.UAS
c                   READCF, SETUP, COMP
c     4)  Add spatial processing method (MSPACE) and code station
c         location for any map projection, but set CF inputs to LL
c         as before (future update may allow others)
c         New:      MAP.UAS, COORDLIB(included)
c                   INTERPH, INTERPHWIND
c         Modified: CONTROL.UAS
c                   BLOCK DATA, READCF, COMP
c     5)  Trap/correct longitude differences that exceed 180 degrees
c         when locating nearest cell.
c         Modified: COMP
c     6)  Add extraction location to output UP.DAT header
c         Modified: COMP, WRHD
c     7)  Previous UP.DAT file used to start run is a formatted file,
c         but was opened unformatted.
c         Modified: SETUP
c     8)  Add ESTRNG for error message reporting, and transfer control
c         back to main program to treat errors (no STOP in subroutines)
c         New:      ERR.CAL
c         Modified: (main), BLOCK DATA, SETUP, READCF,
c                   COMP, GETVER, WRHD
c
c --- v 1.5 Level 050208  ====> v 1.51 Level 080919 (DGS)
c     1)  Move the time zone record so that it appears just
c         before the time information record when station
c         location information is written
c         Modified: WRHD
c
c --- v 1.32 Level 050208  ====> v 1.5 Level 050208 (I. Lee)
c     1)  v1.3(work) of UAMAKE was created concurrently with this version,
c         therefore this version is being renamed to v1.5
c
c --- v 1.31 Level 041216  ====> v 1.32 Level 050208 (I. Lee)
c     1)  Added parameter "MXLEV" to "params.uas" for the maximum number
c         of vertical levels allowed. Previously, it had been hard-coded
c         at 40.
c     2)  Added subroutine GETVER (taken from WDEXTRA 1.10,Level:041111)
c         to "uamake_comp.for" which identifies the type/version of input 
c         3D.DAT file. Allows UAMAKE to run on MM4.DAT, MM5.DAT, M3D 
c         format 3D.DAT files.
c
c --- v 1.3 Level 041122  ====> v 1.31 Level 041216 (I. Lee)
c     1)  Removed Subroutine COMP and Subroutine WRHD and put them in
c         "uamake_comp.for" in order to separate out the main 
c         computational code of UAMAKE.
c         There will now be two sets of FORTRAN code that will call to
c         "uamake_comp.for", one will be "uamake_fortran.for" for
c         creating an internal FORTRAN executable, and the other will be
c         "uamake_dll.for" which will be used to create a VISUAL BASIC
c         DLL for use in a distributed GUI.
c         Now, if UAMAKE needs to be updated or modified, this can 
c         be done by just modifying the code once in "uamake_comp.for". 
c         
c --- V 1.2 Level 041116  ====> V 1.3 Level 041122 (I. Lee)
c     1)  Introduce new time format with beginning/ending times for each 
c         record, including seconds (UP.DAT version 2.1)
c     2)  Changes only to the input/output subroutines, not to the main
c         computational subroutines. Hourly (at most) records only-
c         Changes to subroutines WRHD,READCF,COMP,SUBMISS
c
c --- V 1.1 Level 020330  ====> V 1.2 Level 041116 (I. Lee)
c     1)  Update header format structure to UP.DAT version 2.0
c     2)  Changes made to WRHD
c
c --- V 1.0 Level 001031  ====> V 1.1 Level 020330 (D. Strimaitis)
c     1)  Add upper pressure level to limit heights extracted
c     2)  Add full header output
c     3)  Add standard control-file structure
c     4)  Add scratch file to hold output until final time is known
c-----------------------------------------------------------------------
      Program UAMAKE
c
c --- Include parameters
      include 'params.uas'
c --- Include common blocks
      include 'qa.uas'
      include 'err.cal'
c
c --- Set version and level number of program (stored in /QA/ and
c --- checked against values set in PARAMS.UAS)
      ver='1.53'
      level='090411'
c
c --- SETUP PHASE -- read control file information
      call SETUP
      if(iret.NE.0) then
         klen=LEN_TRIM(estrng)
         write(*,*) estrng(1:klen)
         stop
      endif
c
c --- COMPUTATIONAL PHASE -- process data files
      call UAMAKE_COMP
      if(iret.NE.0) then
         klen=LEN_TRIM(estrng)
         write(*,*) estrng(1:klen)
         stop
      endif
c
c --- TERMINATION PHASE -- program termination functions
      call FIN
c
      stop
      end
c----------------------------------------------------------------------
      BLOCK DATA
c----------------------------------------------------------------------
c
c --- UAMAKE    Version: 1.53           Level: 090411        BLOCK DATA
c               D. Strimaitis
c
c --- Include parameter statements
      include 'params.uas'
c
c --- Include common blocks
      include 'filnam.uas'
      include 'control.uas'
      include 'err.cal'

c --- ERR common block
      data iret/0/
      data estrng/' '/

c --- FILNAM common block
      data runinp/'uamake.inp'/,outfil/'up_uamake.dat'/,
     1     lstfil/'uamake.lst'/,prevfil/'uaprev.dat'/,
     2     nm3d/1/
c --- FILLOG common block
      data lcfiles/.true./

c --- CONTROL common block
c --- Primary variables
      data lprev/.false./
      data idsta/99999/
      data cname/'M3D0'/
      data iptop/500/
      data mspace/1/
      data mtimes/1/
      data hrmask/11*0,1,11*0,1/

      end

c----------------------------------------------------------------------
c --- BRING IN CALPUFF SYSTEM UTILITY SUBROUTINES
      include 'calutils_err.for'
c --- BRING IN MAIN COMPUTATIONAL SUBROUTINE
      include 'uamake_comp_v153.for'
c --- BRING IN COORDINATE TRANSFORMATION SUBROUTINES
      include 'coordlib_err.for'
c----------------------------------------------------------------------

c-----------------------------------------------------------------------
      subroutine setup
c-----------------------------------------------------------------------
c
c --- UAMAKE    Version: 1.53           Level: 090411             SETUP
c               D. Strimaitis
c
c --- PURPOSE:     SETUP reads and checks the control data provided,
c                  sets logicals, echoes the control data, and opens
c                  the data files if inputs are valid.
c
c --- UPDATES:
c
c --- V 1.52 Level 090225  ====> V 1.53 Level 090411 (DGS)
c           - Report spatial and temporal processing methods
c
c --- V 1.3 Level 041122  ====> V 1.52 Level 090225 (DGS)
c           - Filenames changed from c*70 to c*132
c           - /ERR/ processing
c
c --- ARGUMENTS:
c       PASSED:  none
c
c --- RETURNED:  /CONTROL/   data
c
c --- SETUP called by:  (main)
c --- SETUP calls:      READCF
c-----------------------------------------------------------------------
c --- Include file of parameters and commons
      include 'params.uas'
      include 'control.uas'
      include 'filnam.uas'
      include 'qa.uas'
      include 'err.cal'

c --- Get date and time from system
      call DATETM(rdate,rtime,rcpu)

c --- Get the name of the control file from the
c --- command line
      call COMLINE(runinp)

c --- Open the control file
      open(iocnt,file=runinp,status='old')

c --- Report progress
      write(iomesg,*)'SETUP PHASE'

c --- Check that the version and level number in the parameter
c --- file matches those in the code itself
      if(ver.ne.mver.or.level.ne.mlevel)then
         write(iomesg,10) ver,level,mver,mlevel
10       format(/1x,'ERROR in SUBR. SETUP -- The UAMAKE version ',
     1   'and level numbers do not match those in the parameter file'/
     2   5x,'    Model Code - Version: ',a12,' Level: ',a12/
     3   5x,'Parameter File - Version: ',a12,' Level: ',a12)
         stop
      endif

c --- Read control file (open list file)
      call READCF
      if(iret.NE.0) then
         klen=LEN_TRIM(estrng)
         estrng='SETUP:'//estrng(1:klen)
         return
      endif

c --- Open remaining files (not M3D files)
      open(ioout,file=outfil)
      if(LPREV) then
         open(ioprev,file=prevfil,form='formatted',status='old')
      endif
c --- Scratch file
      open(iotmp,status='scratch')

c --- Write header lines to list-file

      write(iolst,*)
      write(iolst,*) '--------------------------'
      write(iolst,*) '    SETUP Information'
      write(iolst,*) '--------------------------'
      write(iolst,*)

c -----------------------------
c --- Report control data
c -----------------------------

      write(iolst,*)
      write(iolst,*) 'Input File Names ------------------'
      k2=LEN_TRIM(runinp)
      write(iolst,'(a)') 'Control   : '//runinp(1:k2)
      do i=1,nm3d
         k2=LEN_TRIM(datafil(i))
         write(iolst,'(a)') 'M3D file  : '//datafil(i)(1:k2)
      enddo   

      if(LPREV) then
         k2=LEN_TRIM(prevfil)
         write(iolst,*)
         write(iolst,'(a)') 'Prev file: ',prevfil(1:k2)
      endif

      write(iolst,*)
      write(iolst,*) 'Output File Names -----------------'
      k2=LEN_TRIM(lstfil)
      write(iolst,'(a)') 'lstfil    : ' ,lstfil(1:k2)
      k2=LEN_TRIM(outfil)
      write(iolst,'(a)') 'outfil    : ' ,outfil(1:k2)

      write(iolst,*)
      write(iolst,*) 'Processing Options ----------------'
      write(iolst,*)
      write(iolst,*) 'Spatial Processing Method?  :  ',mspace
      write(iolst,*) 'Temporal Processing Method? :  ',mtimes
      if(mtimes.EQ.3) then
         write(iolst,*) '  Mask(01-12) = ',(hrmask(k),k=1,12)
         write(iolst,*) '  Mask(13-24) = ',(hrmask(k),k=13,24)
      endif
      write(iolst,*)
      write(iolst,*) 'Continuation Run? :  ',lprev
      write(iolst,6030) iptop
6030  FORMAT(//,'Pressure Levels Extracted:  Surface to ',i6,' mb')


      write(iolst,*)
      write(iolst,*) 'Station data ----------------------'
      write(iolst,*) 'ID    : ' ,idsta
      write(iolst,*) 'Name  : ' ,cname
      write(iolst,*) 'lat(N): ' ,rnlatY
      write(iolst,*) 'lon(E): ' ,relonX
      write(iolst,*)
      write(iolst,*)
      write(iolst,*)

      return
      end
c----------------------------------------------------------------------
      subroutine readcf
c----------------------------------------------------------------------
c
c --- UAMAKE    Version: 1.53           Level: 090225            READCF
c               D. Strimaitis
c
c --- PURPOSE:  Read the control file containing the file names of
c               the input and output files of the run, and associated
c               control variables, place into program variables,
c               and QA information.
c
c --- UPDATES:
c
c --- V 1.52 Level 090225  ====> V 1.53 Level 090411 (DGS)
c           - Add temporal processing method (MTIMES and HRMASK)
c
c --- V 1.3 Level 041122  ====> V 1.52 Level 090225 (DGS)
c           - Filenames changed from c*70 to c*132
c           - Add spatial processing method (MSPACE)
c           - Set station location projection type to LL (WGS-84)
c           - /ERR/ error processing
c
c --- INPUTS:
c
c ---    Common block /QA/ variables:
c           VER, LEVEL
c
c        Parameters: IOCNT, IOLST, IOMESG, IOINP, MXVAR, MXFIL
c
c --- OUTPUT:
c
c ---    Common block /FILNAM/ variables:
c           nm3d,lstfil,outfil,prevfil,
c           datafil(mxfil),justname(mxfil),
c           lcfiles
c ---    Common block /CONTROL/ variables:
c           lprev,idsta,mspace,mtimes,hrmask,
c           iptop,rnlatY,relonX,cname,clat,clon
c ---    Common block /MAP/ variables:
c           iutmzn,feast,fnorth,rnlat0,relon0,rnlat1,rnlat2,
c           datum,cmap,utmhem,clat0,clon0,clat1,clat2
c
c --- READCF called by:  SETUP
c --- READCF calls:      READIN, FILCASE, XTRACTLL
c----------------------------------------------------------------------
c
c --- Include parameter statements and commons
      include 'params.uas'
      include 'params.cal'
c
c --- Include common blocks
      include 'control.uas'
      include 'filnam.uas'
      include 'map.uas'
      include 'qa.uas'
      include 'err.cal'
c
c --- Local variables
      character*4 ctemp(132,3)
      character*132 m3dfil
      character*4 clatlon(16,2),cname4(4)
      character*12 cvdic(mxvar,3)
      integer ivleng(mxvar,3),ivtype(mxvar,3)
      logical lecho, lerrcf

c --- Initialize local variables
      data lecho/.false./, lerrcf/.false./
      data names/3/

c --- Set Dictionary

      data cvdic/
     a  'OUTFIL','LSTFIL','PREVFIL','LCFILES','NM3D',  55*' ',
     2  'M3D',  59*' ',
     b  'LPREV','IDSTA','CNAME','LAT','LON','PSTOP','MSPACE',
     b  'MTIMES','HRMASK', 51* ' '/

      data ivleng/
     a  3*132,2*1,         55*0,
     2  132,               59*0,
     b  2*1,4,2*16,3*1,24, 51*0/

c --- Variable types (ivtype) are coded as:
c          0 = null
c          1 = real
c          2 = integer
c          3 = logical
c          4 = character
      data ivtype/
     a  3*4,3,2,       55*0,
     2  4,             59*0,
     b  3,2,3*4,1,3*2, 51*0/

c ------------------
c --- Input Group 0a
c ------------------

c --- Initialize the temporary arrays
      do i=1,names
         do j=1,132
            ctemp(j,i)(1:1)=' '
         enddo
      enddo

c --- Read the group data
       call READIN(cvdic(1,1),ivleng(1,1),ivtype(1,1),iocnt,iomesg,
     & lecho,
     1 ctemp(1,1),ctemp(1,2),ctemp(1,3),lcfiles,nm3d,
     2 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6 idum,idum,idum,idum,idum,idum,idum)

      if(iret.NE.0) then
         klen=LEN_TRIM(estrng)
         estrng='READCF:'//estrng(1:klen)
         return
      endif

c --- Prepare any filenames included in the I/O file by erasing
c --- the default filename set above
      if(ctemp(1,1)(1:1).ne.' ')outfil=' '
      if(ctemp(1,2)(1:1).ne.' ')lstfil=' '
      if(ctemp(1,3)(1:1).ne.' ')prevfil=' '

c --- Transfer the char*4 data into the char*132 variables
      do j=1,132
         if(ctemp(j,1)(1:1).ne.' ')outfil(j:j)=ctemp(j,1)(1:1)
         if(ctemp(j,2)(1:1).ne.' ')lstfil(j:j)=ctemp(j,2)(1:1)
         if(ctemp(j,3)(1:1).ne.' ')prevfil(j:j)=ctemp(j,3)(1:1)
      enddo

c --- Convert the file names to the proper case
      call FILCASE(lcfiles,outfil)
      call FILCASE(lcfiles,lstfil)
      call FILCASE(lcfiles,prevfil)

c --- Open listfile
      open(iolst,file=lstfil,status='unknown')

c --- Write banner to list file
      write(iolst,5) ver,level
5     format(///,26x,'UAMAKE OUTPUT SUMMARY',/,19x,'VERSION:  ',A8,
     1       ' LEVEL:  ',A8///)

c ------------------
c --- Input Group 0b
c ------------------

      do k=1,nm3d
c ---    Initialize the temporary array for the file name
         do j=1,132
            ctemp(j,1)(1:1)=' '
            m3dfil(j:j)=' '
         enddo

c ---    Read one M3D filename entry
      call READIN(cvdic(1,2),ivleng(1,2),ivtype(1,2),iocnt,iolst,lecho,
     1 ctemp(1,1),
     2 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     3 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)

         if(iret.NE.0) then
            klen=LEN_TRIM(estrng)
            estrng='READCF:'//estrng(1:klen)
            return
         endif

c ---    Process if filename array has room for this entry
         if(k.LE.mxfil) then

c ---       Transfer the char*4 data into the char*132 variable
            do j=1,132
               if(ctemp(j,1)(1:1).ne.' ')m3dfil(j:j)=ctemp(j,1)(1:1)
            enddo

c ---       Convert the file name to the proper case
            call FILCASE(lcfiles,m3dfil)         

c ---       Place information in M3D name array, up to MXFIL
            datafil(k)=m3dfil

c ---       Extract just file name without path info.
            ipath=0
            ilen=0
            do n=1,132
                if(m3dfil(n:n).NE.' ') ilen=n
                if(m3dfil(n:n).EQ.'/' .OR. m3dfil(n:n).EQ.'\') ipath=n
            enddo
            read(m3dfil(ipath+1:ilen),'(a)') justname(k)

         endif
      enddo

c -----------------
c --- Input Group 1
c -----------------

c --- Initialize the temporary arrays for the character fields
      do i=1,2
         do j=1,16
            clatlon(j,i)(1:1)=' '
         enddo
      enddo
      do j=1,16
         clat(j:j)=' '
         clon(j:j)=' '
      enddo
      do j=1,4
         cname4(j)(1:1)=' '
         cname(j:j)=' '
      enddo

      call READIN(cvdic(1,3),ivleng(1,3),ivtype(1,3),iocnt,iolst,lecho,
     1 LPREV,IDSTA,CNAME4,CLATLON(1,1),CLATLON(1,2),PSTOP,MSPACE,
     2 MTIMES,HRMASK,
     3 idum,idum,idum,
     4 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     5 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     6 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,
     7 idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum,idum)

      if(iret.NE.0) then
         klen=LEN_TRIM(estrng)
         estrng='READCF:'//estrng(1:klen)
         return
      endif

c --- Store pressure at sounding top as integer
      iptop=NINT(pstop)

c --- Transfer the name array data into the char*4 variable
      do j=1,4
         if(cname4(j)(1:1).ne.' ')cname(j:j)=cname4(j)(1:1)
      enddo

c --- Transfer the char*4 data into the char*16 variables
      do j=1,16
         if(clatlon(j,1)(1:1).ne.' ')clat(j:j)=clatlon(j,1)(1:1)
         if(clatlon(j,2)(1:1).ne.' ')clon(j:j)=clatlon(j,2)(1:1)
      enddo

c --- Translate character lat/lon to real NLat/ELon
      if(clat(1:1).NE.' ') call XTRACTLL(iolst,'LAT ',clat,rnlatY)
      if(clon(1:1).NE.' ') call XTRACTLL(iolst,'LON ',clon,relonX)

c ---------------------
c --- Perform QA checks
c ---------------------

c --- Test for valid NM3D
      if(nm3d.GT.mxfil) then
         write(iolst,*)
         write(iolst,*) 'READCF:  Error in Input Group 0a'
         write(iolst,*) 'NM3D exceeds the parameter MXFIL '
         write(iolst,*) 'NM3D, MXFIL = ',nm3d,mxfil
         write(iolst,*) 'Increase MXFIL in PARAMS.UAS and recompile'
         write(iolst,*) 'or reduce the number of M3D files'
         lerrcf=.TRUE.
      endif

c --- Test for valid MSPACE
      if(mspace.LT.1 .OR. mspace.GT.2) then
         write(iolst,*)
         write(iolst,*) 'READCF:  Error in Input Group 1'
         write(iolst,*) 'Invalid Spatial Processing Method'
         write(iolst,*) '  Expected MSPACE = 1,2'
         write(iolst,*) '  Found    MSPACE = ',mspace
         lerrcf=.TRUE.
      endif

c --- Test for valid MTIMES
      if(mtimes.LT.1 .OR. mtimes.GT.3) then
         write(iolst,*)
         write(iolst,*) 'READCF:  Error in Input Group 1'
         write(iolst,*) 'Invalid Temporal Processing Method'
         write(iolst,*) '  Expected MTIMES = 1,2,3'
         write(iolst,*) '  Found    MTIMES = ',mtimes
         lerrcf=.TRUE.
      endif

c --- Test for valid HRMASK
      ibad=0
      do k=1,24
         if(hrmask(k).LT.0 .OR. hrmask(k).GT.1) ibad=1
      enddo
      if(ibad.EQ.1) then
         write(iolst,*)
         write(iolst,*) 'READCF:  Error in Input Group 1'
         write(iolst,*) 'Invalid Temporal Processing Mask'
         write(iolst,*) '  Expected HRMASK = 0 or 1 (24 values)'
         write(iolst,*) '  Found    HRMASK = ',hrmask
         if(mtimes.EQ.3) lerrcf=.TRUE.
      endif

c --- STOP now if error exists in the control file
      if(LERRCF) then
c         write(*,*)'ERRORS are found in the CONTROL file'
c         write(*,*)'Review messages written to the LIST file'
c         stop
         iret=1
         estrng='Error in READCF -- see list file'
         return
      endif

c --- Set map projection information for coordinate system used to
c --- identify station location
      cmap='LL      '
      datum='WGS-84  '
c --- Remaining are not used for LL
      utmhem='N   '
      iutmzn=0
      feast=0.0
      fnorth=0.0
      rnlat0=0.0
      relon0=0.0
      rnlat1=0.0
      rnlat2=0.0
      clat0='0.0N            '
      clon0='0.0E            '
      clat1='0.0N            '
      clat2='0.0N            '

      return
      end

c----------------------------------------------------------------------
      subroutine fin
c----------------------------------------------------------------------
c
c --- UAMAKE   Version: 1.53            Level: 041122               FIN
c ---          J. Scire, Earth Tech
c
c --- PURPOSE:  Run termination routine -- compute runtime
c
c --- INPUTS:
c       Common block /QA/
c          rdate, rtime, rcpu
c       Parameters: IOLST, IOMESG
c
c --- OUTPUT:  none
c
c --- FIN called by:  MAIN
c --- FIN calls:      DATETM, JULDAY, DELTT
c----------------------------------------------------------------------
c
c --- include parameters
      include 'params.uas'
      include 'qa.uas'
c
      character*8 rtime2
      character*10 rdate2
c
      write(iomesg,*)'TERMINATION PHASE'
c
c --- get system date & time at end of run
      call datetm(rdate2,rtime2,rcpu)
c
c --- compute runtime
      read(rtime(1:2),10)ihr1
      read(rtime(4:5),10)imin1
      read(rtime(7:8),10)isec1
10    format(i2)
      t1=ihr1*3600.+imin1*60.+isec1
c
      read(rtime2(1:2),10)ihr2
      read(rtime2(4:5),10)imin2
      read(rtime2(7:8),10)isec2
      t2=ihr2*3600.+imin2*60.+isec2
c
      if(rdate.eq.rdate2)then
         delt=t2-t1
      else
         read(rdate(1:2),10)imo1
         read(rdate(4:5),10)iday1
         read(rdate(7:10),'(i4)')iyr1
         call julday(iolst,iyr1,imo1,iday1,ijul1)
c
         read(rdate2(1:2),10)imo2
         read(rdate2(4:5),10)iday2
         read(rdate2(7:10),'(i4)')iyr2
         call julday(iomesg,iyr2,imo2,iday2,ijul2)
c
c ---    compute no. hours from beg. of first hour of run to
c ---    ending hour of ending day of the run
         call deltt(iyr1,ijul1,ihr1,iyr2,ijul2,ihr2,idelhr)
c
c ---    adjust for minutes and seconds
         delt=idelhr*3600.-imin1*60.-isec1+imin2*60.+isec2
      endif

c --- On the PC, the runtime and CPU time are the same
c --- (DATETM provides RCPU = 0.0 on the PC)
      if(rcpu.EQ.0.0)rcpu=delt

c --- Report current date
      write(iolst,1402)rtime2,rdate2,delt,rcpu
1402  format(//2x,'End of run -- Clock time: ',a8/
     1         2x,'                    Date: ',a10//
     2         2x,'      Elapsed Clock Time: ',f10.1,' (seconds)'//
     3         2x,'                CPU Time: ',f10.1,' (seconds)')

c
      return
      end
